@@ -3,40 +3,61 @@ from copy import copy
 
 from django.conf import settings
 from django import template
+from django.core.cache import cache
 from django.db.models import signals
 from django.utils.http import urlquote
 from django.template.defaulttags import url as url_tag
 
 from models import Tree, TreeItem
 
+# Sitetree objects are stored in Django cache for a year (60 * 60 * 24 * 365 = 31536000 sec).
+# Cache is only invalidated on sitetree or sitetree item change.
+CACHE_TIMEOUT = 31536000
+
 
 class SiteTree(object):
 
     def __init__(self):
+        self.cache = None
         # This points to global sitetree context.
         self.global_context = None
-        # Cache. This stores all requested sitetrees with items.
-        self.cache_sitetrees = {}
-        # Cache. Current tree item - tree item resolved for current page.
-        self.cache_current_item = {}
-        # Cache. Relations between item title and its rendered name.
-        self.cache_titles = {}
-        # Cache. Relations between item url and its resolved form.
-        self.cache_urls = {}
-        # Cache. Breadcrumb path.
-        self.cache_breadcrumbs = {}
-        # Cache. Tree structures as lists.
-        self.cache_treestruct = {}
-        # Cache. Parents mappings.
-        self.cache_parents = {}
-        # Cache. Utility dictionary with site tree items indexed by IDs.
-        self.cache_items_by_ids = {}
         # Listen for signals from the models.
-        signals.post_save.connect(self.cache_flush_tree, sender=Tree)
-        signals.post_save.connect(self.cache_flush_tree, sender=TreeItem)
-        signals.post_delete.connect(self.cache_flush_tree, sender=TreeItem)
+        signals.post_save.connect(self.cache_empty, sender=Tree)
+        signals.post_save.connect(self.cache_empty, sender=TreeItem)
+        signals.post_delete.connect(self.cache_empty, sender=TreeItem)
         # Listen to the changes in item permissions table.
-        signals.m2m_changed.connect(self.cache_flush_tree, sender=TreeItem.access_permissions)
+        signals.m2m_changed.connect(self.cache_empty, sender=TreeItem.access_permissions)
+
+    def cache_init(self):
+        """Initializes local cache from Django cache."""
+        cache_ = cache.get('sitetrees')
+        if cache_ is None:
+            # Init cache dictionary with predefined enties.
+            cache_ = {'sitetrees': {}, 'urls': {}, 'parents': {}, 'items_by_ids': {}}
+        self.cache = cache_
+
+    def cache_save(self):
+        """Saves sitetree data to Django cache."""
+        cache.set('sitetrees', self.cache, CACHE_TIMEOUT)
+
+    def cache_empty(self, **kwargs):
+        """Empties cached sitetree data."""
+        self.cache = False
+        cache.delete('sitetrees')
+
+    def get_cache_entry(self, entry_name, key):
+        """Returns cache entry parameter value by its name."""
+        return self.cache[entry_name].get(key, False)
+
+    def update_cache_entry_value(self, entry_name, key, value):
+        """Updates cache entry parameter with new data."""
+        if key not in self.cache[entry_name]:
+            self.cache[entry_name][key] = {}
+        self.cache[entry_name][key].update(value)
+
+    def set_cache_entry(self, entry_name, key, value):
+        """Replaces entire cache entry parameter data by its name with new data."""
+        self.cache[entry_name][key] = value
 
     def set_global_context(self, context):
         """Saves context as global context if not already set or if changed.
@@ -45,7 +66,6 @@ class SiteTree(object):
         """
         if not self.global_context or hash(context) != hash(self.global_context):
             self.global_context = context
-            self.cache_flush_contextual_data()
 
     def get_sitetree(self, alias):
         """Gets site tree items from the given site tree.
@@ -53,33 +73,57 @@ class SiteTree(object):
         Returns items.
 
         """
-        if alias in self.cache_sitetrees:
-            sitetree = self.cache_sitetrees[alias]
-        else:
-            sitetree = self.cache_sitetrees[alias] = TreeItem.objects.select_related('parent', 'tree').filter(
-                tree__alias__exact=alias).order_by('parent', 'sort_order')
+        self.cache_init()
+        sitetree_needs_caching = False
+        sitetree = self.get_cache_entry('sitetrees', alias)
+        if not sitetree:
+            sitetree = TreeItem.objects.select_related('parent', 'tree').\
+                   filter(tree__alias__exact=alias).order_by('parent', 'sort_order')
+            self.set_cache_entry('sitetrees', alias, sitetree)
+            sitetree_needs_caching = True
 
-        # Create shortcuts dictionary.
-        self.link_items_to_ids(alias)
+        parents = self.get_cache_entry('parents', alias)
+        if not parents:
+            parents = defaultdict(list)
+            for item in sitetree:
+                parent = getattr(item, 'parent')
+                parents[parent].append(item)
+            self.set_cache_entry('parents', alias, parents)
 
-        for sitetree_item in sitetree:
-            if not hasattr(sitetree_item, 'depth'):
-                sitetree_item.depth = self.calculate_item_depth(alias, sitetree_item.id)
-            sitetree_item.depth_range = range(sitetree_item.depth)
-            # Resolve item's URL.
-            sitetree_item.url_resolved = self.url(sitetree_item)
-            # Resolve item permissions.
-            if sitetree_item.access_restricted:
-                sitetree_item.perms = set([u'%s.%s' % (perm.content_type.app_label, perm.codename) for perm in
-                                           sitetree_item.access_permissions.select_related()])
+        for item in sitetree:
+
+            if sitetree_needs_caching:
+                # Prepare items by ids cache.
+                self.update_cache_entry_value('items_by_ids', alias, {item.id: item})
+
+                if item in parents:
+                    item.has_children = True
+                else:
+                    item.has_children = False
+
+                if not hasattr(item, 'depth'):
+                    item.depth = self.calculate_item_depth(alias, item.id)
+                item.depth_range = range(item.depth)
+
+                # Resolve item permissions.
+                if item.access_restricted:
+                    item.perms = set([u'%s.%s' % (perm.content_type.app_label, perm.codename) for perm in
+                                               item.access_permissions.select_related()])
+            # Contextual properties.
+            item.url_resolved = self.url(item)
+            item.title_resolved = item.title
+            item.is_current = False
+            item.in_current_branch = False
 
-        # Generate parents mappings.
-        self.get_parents(alias)
         # Get current item for the given sitetree.
         self.get_tree_current_item(alias)
         # Parse titles.
         self.parse_titles(sitetree)
 
+        # Save sitetree data into cache if needed.
+        if sitetree_needs_caching:
+            self.cache_save()
+
         return sitetree
 
     def calculate_item_depth(self, tree_alias, item_id, depth=0):
@@ -92,37 +136,9 @@ class SiteTree(object):
             depth = item.depth + 1
         return depth
 
-    def link_items_to_ids(self, tree_alias):
-        """Creates utility shortcuts dictionary where tree items indexed by IDs."""
-        if tree_alias not in self.cache_items_by_ids:
-            self.cache_items_by_ids[tree_alias] = {}
-            for item in self.cache_sitetrees[tree_alias]:
-                self.cache_items_by_ids[tree_alias][item.id] = item
-
     def get_item_by_id(self, tree_alias, item_id):
         """Get the item from the tree by its ID."""
-        return self.cache_items_by_ids[tree_alias][item_id]
-
-    def get_parents(self, tree_alias):
-        """Creates parents mappings."""
-        if tree_alias not in self.cache_parents:
-            # Group items under parents
-            parents = defaultdict(list)
-
-            for item in self.cache_sitetrees[tree_alias]:
-                item.has_children = False
-                parent = getattr(item, 'parent')
-                parents[parent].append(item)
-            self.cache_parents[tree_alias] = parents
-
-            for item in self.cache_sitetrees[tree_alias]:
-                if item in parents:
-                    item.has_children = True
-
-        else:
-            parents = self.cache_parents[tree_alias]
-
-        return parents
+        return self.get_cache_entry('items_by_ids', tree_alias)[item_id]
 
     def get_tree_current_item(self, tree_alias):
         """Resolves current tree item of 'tree_alias' tree matching current
@@ -131,21 +147,17 @@ class SiteTree(object):
         """
         current_item = None
 
-        if tree_alias in self.cache_current_item:
-            current_item = self.cache_current_item[tree_alias]
+        if 'request' not in self.global_context and self.global_context.current_app != 'admin':
+            raise SiteTreeError('Sitetree needs "django.core.context_processors.request" to be in TEMPLATE_CONTEXT_PROCESSORS in your settings file. If it is, check that your view pushes request data into the template.')
         else:
-            if 'request' not in self.global_context and self.global_context.current_app != 'admin':
-                raise SiteTreeError('Sitetree needs "django.core.context_processors.request" template processor to be enabled. Please add it to TEMPLATE_CONTEXT_PROCESSORS in your settings file.')
-            else:
-                # urlquote is a try to support non-ascii in url.
-                current_url = urlquote(self.global_context['request'].path)
-                if tree_alias in self.cache_urls:
-                    for url_item in self.cache_urls[tree_alias]:
-                        self.cache_urls[tree_alias][url_item][1].is_current = False
-                        if self.cache_urls[tree_alias][url_item][0] == current_url:
-                            current_item = self.cache_urls[tree_alias][url_item][1]
-
-            self.cache_current_item[tree_alias] = current_item
+            # urlquote is a try to support non-ascii in url.
+            current_url = urlquote(self.global_context['request'].path)
+            urls_cache = self.get_cache_entry('urls', tree_alias)
+            if urls_cache:
+                for url_item in urls_cache:
+                    urls_cache[url_item][1].is_current = False
+                    if urls_cache[url_item][0] == current_url:
+                        current_item = urls_cache[url_item][1]
 
         if current_item is not None:
             current_item.is_current = True
@@ -158,7 +170,7 @@ class SiteTree(object):
         'sitetree_item' points to TreeItem object, 'url' property of which
             is processed as URL pattern or simple URL.
 
-        'tag_arguments' id a list of additional arguments passed to
+        'tag_arguments' is a list of additional arguments passed to
             'sitetree_url' in template.
 
         """
@@ -180,15 +192,24 @@ class SiteTree(object):
                 resolved = self.resolve_var(view_argument)
                 # In case of non-ascii data we leave variable unresolved.
                 if isinstance(resolved, unicode):
-                    if resolved.encode("ascii", "ignore").decode("ascii") != resolved:
+                    if resolved.encode('ascii', 'ignore').decode('ascii') != resolved:
                         resolved = view_argument
 
                 view_arguments.append(resolved)
 
             # URL parameters from site tree item should be concatenated with
             # those from template.
-            view_arguments = tag_arguments + view_arguments
-            view_arguments = [str(view_argument) for view_argument in view_arguments]
+            arguments_couled = tag_arguments + view_arguments
+            view_arguments = []
+
+            for argument in arguments_couled:
+                argument = str(argument)
+                # To be able to process slug-like strings (strings with "-"'s and "_"'s)
+                # we enclose those in double quotes.
+                if '-' in argument or '_':
+                    argument = '"%s"' % argument
+                view_arguments.append(argument)
+            
             view_arguments = ' '.join(view_arguments).strip()
             view_path = view_path[0]
             url_pattern = u'%s %s' % (view_path, view_arguments)
@@ -199,11 +220,14 @@ class SiteTree(object):
 
         # Create 'cache_urls' for this tree.
         tree_alias = sitetree_item.tree.alias
-        if tree_alias not in self.cache_urls:
-            self.cache_urls[tree_alias] = {}
 
-        if url_pattern in self.cache_urls[tree_alias]:
-            resolved_url = self.cache_urls[tree_alias][url_pattern][0]
+        entry_from_cache = self.get_cache_entry('urls', tree_alias)
+        if not entry_from_cache:
+            entry_from_cache = {}
+            self.set_cache_entry('urls', tree_alias, {})
+
+        if url_pattern in entry_from_cache:
+            resolved_url = entry_from_cache[url_pattern][0]
         else:
             if sitetree_item.urlaspattern:
                 # Form token to pass to Django 'url' tag.
@@ -219,7 +243,8 @@ class SiteTree(object):
             else:
                 resolved_url = url_pattern
 
-            self.cache_urls[tree_alias][url_pattern] = (resolved_url, sitetree_item)
+            self.update_cache_entry_value('urls', tree_alias, {url_pattern: (resolved_url, sitetree_item)})
+
         return resolved_url
 
     def init_tree(self, tree_alias, context):
@@ -264,6 +289,7 @@ class SiteTree(object):
         parent_aliases = []
 
         current_item = self.get_tree_current_item(tree_alias)
+        self.tree_climber(tree_alias, current_item)
 
         for branch_id in tree_branches.split(','):
             branch_id = branch_id.strip()
@@ -321,10 +347,12 @@ class SiteTree(object):
         if not sitetree_items:
             return ''
         current_item = self.get_tree_current_item(tree_alias)
-        trail_items = []
+
+        self.cache_breadcrumbs = []
         if current_item is not None:
-            trail_items = self.get_breadcrumbs(tree_alias, current_item)
-        return trail_items
+            self.breadcrumbs_climber(tree_alias, current_item)
+            self.cache_breadcrumbs.reverse()
+        return self.cache_breadcrumbs
 
     def tree(self, tree_alias, context):
         """Builds and returns tree structure for 'sitetree_tree' tag."""
@@ -347,6 +375,8 @@ class SiteTree(object):
         tree_alias = self.resolve_var(tree_alias)
         # Get tree.
         self.get_sitetree(tree_alias)
+        # Mark path to current item.
+        self.tree_climber(tree_alias, self.get_tree_current_item(tree_alias))
         tree_items = self.get_children(tree_alias, parent_item)
         tree_items = self.filter_items(tree_items, navigation_type)
         my_template = template.loader.get_template(use_template)
@@ -354,7 +384,7 @@ class SiteTree(object):
         return my_template.render(my_context)
 
     def get_children(self, tree_alias, item):
-        return self.cache_parents[tree_alias][item]
+        return self.get_cache_entry('parents', tree_alias)[item]
 
     def filter_items(self, items, navigation_type=None):
         """Filters site tree item's children if hidden and by navigation type.
@@ -365,24 +395,10 @@ class SiteTree(object):
         if self.global_context.current_app != 'admin':
             for item in items:
                 if item.hidden == True ^ (not self.check_access(item, self.global_context)) ^ (
-                navigation_type is not None and getattr(item, 'in' + navigation_type) != True):
+                navigation_type is not None and getattr(item, 'in' + navigation_type, False) != True):
                     items.remove(item)
         return items
 
-    def get_breadcrumbs(self, tree_alias, start_item):
-        """Returns breadcrumb path to the given 'start_item' site tree item."""
-        if tree_alias not in self.cache_breadcrumbs:
-            self.cache_breadcrumbs[tree_alias] = []
-
-        if self.cache_breadcrumbs[tree_alias]:
-            breadcrumbs = self.cache_breadcrumbs[tree_alias]
-        else:
-            self.breadcrumbs_climber(tree_alias, start_item)
-            self.cache_breadcrumbs[tree_alias].reverse()
-            breadcrumbs = self.cache_breadcrumbs[tree_alias]
-
-        return breadcrumbs
-
     def get_ancestor_item(self, tree_alias, start_from):
         """Climbs up the site tree to resolve root item for chosen one."""
         parent = None
@@ -395,11 +411,18 @@ class SiteTree(object):
 
         return parent
 
+    def tree_climber(self, tree_alias, start_from):
+        """Climbs up the site tree to mark items of current branch."""
+        if start_from is not None:
+            start_from.in_current_branch = True
+            if hasattr(start_from, 'parent') and start_from.parent is not None:
+                self.tree_climber(tree_alias, self.get_item_by_id(tree_alias, start_from.parent.id))
+
     def breadcrumbs_climber(self, tree_alias, start_from):
         """Climbs up the site tree to build breadcrumb path."""
         if start_from.inbreadcrumbs and start_from.hidden == False and self.check_access(start_from,
                                                                                          self.global_context):
-            self.cache_breadcrumbs[tree_alias].append(start_from)
+            self.cache_breadcrumbs.append(start_from)
         if hasattr(start_from, 'parent') and start_from.parent is not None:
             self.breadcrumbs_climber(tree_alias, self.get_item_by_id(tree_alias, start_from.parent.id))
 
@@ -436,41 +459,19 @@ class SiteTree(object):
             context = self.global_context
 
         for item in items:
-            if not hasattr(item, 'title_resolved'):
-                item.title_resolved = item.title
+            if item.title.find(template.VARIABLE_TAG_START) != -1:
+                my_lexer = template.Lexer(item.title, template.UNKNOWN_SOURCE)
+                my_tokens = my_lexer.tokenize()
 
-            if item.title in self.cache_titles:
-                item.title_resolved = self.cache_titles[item.title]
-            else:
-                if item.title.find(template.VARIABLE_TAG_START) != -1:
-                    my_lexer = template.Lexer(item.title, template.UNKNOWN_SOURCE)
-                    my_tokens = my_lexer.tokenize()
+                for my_token in my_tokens:
+                    if my_token.token_type not in (template.TOKEN_TEXT, template.TOKEN_VAR):
+                        my_tokens.remove(my_token)
 
-                    for my_token in my_tokens:
-                        if my_token.token_type not in (template.TOKEN_TEXT, template.TOKEN_VAR):
-                            my_tokens.remove(my_token)
+                my_parser = template.Parser(my_tokens)
+                item.title_resolved = my_parser.parse().render(context)
 
-                    my_parser = template.Parser(my_tokens)
-                    item.title_resolved = my_parser.parse().render(context)
-                self.cache_titles[item.title] = item.title_resolved
         return items
 
-    def cache_flush_contextual_data(self, **kwargs):
-        """Flushes cached site tree items titles data."""
-        self.cache_current_item = {}
-        self.cache_titles = {}
-        self.cache_breadcrumbs = {}
-
-    def cache_flush_tree(self, **kwargs):
-        """Flushes cached site tree data."""
-        self.cache_sitetrees = {}
-        self.cache_current_item = {}
-        self.cache_titles = {}
-        self.cache_breadcrumbs = {}
-        self.cache_urls = {}
-        self.cache_parents = {}
-        self.cache_items_by_ids = {}
-
 
 class SiteTreeError(Exception):
     """Exception class for sitetree application."""
